@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2020 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2022 Joe Qian <joeqian@google.com>
  *
  * SPDX-License-Identifier: LGPL-2.1+
  */
@@ -9,6 +10,8 @@
 #include <string.h>
 
 #include "jcat-blob-private.h"
+#include "jcat-bt-checkpoint-private.h"
+#include "jcat-bt-verifier-private.h"
 #include "jcat-common-private.h"
 #include "jcat-context.h"
 #include "jcat-engine-private.h"
@@ -210,6 +213,7 @@ jcat_sha1_engine_func(void)
 	g_autoptr(JcatBlob) blob_sig2 = NULL;
 	g_autoptr(JcatContext) context = jcat_context_new();
 	g_autoptr(JcatEngine) engine = NULL;
+	g_autoptr(JcatEngine) engine_none = NULL;
 	g_autoptr(JcatResult) result_fail = NULL;
 	g_autoptr(JcatResult) result_pass = NULL;
 	const gchar *sig_actual = "7c0ae84b191822bcadbdcbe2f74a011695d783c7";
@@ -254,9 +258,9 @@ jcat_sha1_engine_func(void)
 
 	/* not supported */
 	jcat_context_blob_kind_disallow(context, JCAT_BLOB_KIND_SHA1);
-	engine = jcat_context_get_engine(context, JCAT_BLOB_KIND_SHA1, &error);
+	engine_none = jcat_context_get_engine(context, JCAT_BLOB_KIND_SHA1, &error);
 	g_assert_error(error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED);
-	g_assert_null(engine);
+	g_assert_null(engine_none);
 }
 
 static void
@@ -854,6 +858,76 @@ jcat_context_verify_item_sign_func(void)
 #endif
 }
 
+static void
+jcat_context_verify_item_target_func(void)
+{
+#ifdef ENABLE_PKCS7
+	JcatResult *result;
+	g_autofree gchar *fn_pass = NULL;
+	g_autofree gchar *fn_sig = NULL;
+	g_autofree gchar *pki_dir = NULL;
+	g_autoptr(GBytes) data_fwbin = NULL;
+	g_autoptr(GBytes) data_sig = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(JcatBlob) blob_pkcs7 = NULL;
+	g_autoptr(JcatBlob) blob_target_sha256 = NULL;
+	g_autoptr(JcatItem) item = jcat_item_new("filename.bin");
+	g_autoptr(JcatItem) item_target = jcat_item_new("filename.bin");
+	g_autoptr(JcatContext) context = jcat_context_new();
+	g_autoptr(JcatEngine) engine_sha256 = NULL;
+	g_autoptr(GPtrArray) results_fail = NULL;
+	g_autoptr(GPtrArray) results_pass = NULL;
+
+	/* set up context */
+	jcat_context_set_keyring_path(context, "/tmp");
+	pki_dir = g_test_build_filename(G_TEST_BUILT, "pki", NULL);
+	jcat_context_add_public_keys(context, pki_dir);
+
+	/* get all engines */
+	engine_sha256 = jcat_context_get_engine(context, JCAT_BLOB_KIND_SHA256, &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(engine_sha256);
+
+	/* add SHA256 hash as a target blob */
+	fn_pass = g_test_build_filename(G_TEST_DIST, "colorhug", "firmware.bin", NULL);
+	data_fwbin = jcat_get_contents_bytes(fn_pass, &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(data_fwbin);
+	blob_target_sha256 =
+	    jcat_engine_self_sign(engine_sha256, data_fwbin, JCAT_SIGN_FLAG_NONE, &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(blob_target_sha256);
+	jcat_item_add_blob(item_target, blob_target_sha256);
+
+	/* create the item to verify, with a checksum and the PKCS#7 signature *of the hash* */
+	jcat_item_add_blob(item, blob_target_sha256);
+	fn_sig = g_test_build_filename(G_TEST_BUILT, "colorhug", "firmware.bin.sha256.p7c", NULL);
+	data_sig = jcat_get_contents_bytes(fn_sig, &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(data_sig);
+	blob_pkcs7 = jcat_blob_new_full(JCAT_BLOB_KIND_PKCS7, data_sig, JCAT_BLOB_FLAG_IS_UTF8);
+	jcat_blob_set_target(blob_pkcs7, JCAT_BLOB_KIND_SHA256);
+	jcat_item_add_blob(item, blob_pkcs7);
+
+	/* enforce a checksum and signature match */
+	results_pass = jcat_context_verify_target(context,
+						  item_target,
+						  item,
+						  JCAT_VERIFY_FLAG_DISABLE_TIME_CHECKS |
+						      JCAT_VERIFY_FLAG_REQUIRE_CHECKSUM |
+						      JCAT_VERIFY_FLAG_REQUIRE_SIGNATURE,
+						  &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(results_pass);
+	g_assert_cmpint(results_pass->len, ==, 2);
+	result = g_ptr_array_index(results_pass, 1);
+	g_assert_cmpint(jcat_result_get_timestamp(result), >=, 1502871248);
+	g_assert_cmpstr(jcat_result_get_authority(result), ==, "O=Hughski Limited");
+#else
+	g_test_skip("no GnuTLS support enabled");
+#endif
+}
+
 static void
 jcat_context_verify_item_csum_func(void)
 {
@@ -934,18 +1008,168 @@ jcat_context_verify_item_csum_func(void)
 #endif
 }
 
+static void
+jcat_bits_func(void)
+{
+	g_assert_cmpint(jcat_bits_ones_count64(0), ==, 0);
+	g_assert_cmpint(jcat_bits_ones_count64(1), ==, 1);
+	g_assert_cmpint(jcat_bits_ones_count64(5), ==, 2);
+	g_assert_cmpint(jcat_bits_ones_count64(5), ==, 2);
+	g_assert_cmpint(jcat_bits_ones_count64(0x8000000000000000), ==, 1);
+	g_assert_cmpint(jcat_bits_ones_count64(0xFFFFFFFFFFFFFFFF), ==, 64);
+
+	g_assert_cmpint(jcat_bits_trailing_zeros64(0), ==, 64);
+	g_assert_cmpint(jcat_bits_trailing_zeros64(8), ==, 3);
+	g_assert_cmpint(jcat_bits_trailing_zeros64(24), ==, 3);
+	g_assert_cmpint(jcat_bits_trailing_zeros64(25), ==, 0);
+	g_assert_cmpint(jcat_bits_trailing_zeros64(0x8000000000000000), ==, 63);
+	g_assert_cmpint(jcat_bits_trailing_zeros64(0xFFFFFFFFFFFFFFFF), ==, 0);
+
+	g_assert_cmpint(jcat_bits_length64(0), ==, 0);
+	g_assert_cmpint(jcat_bits_length64(1), ==, 1);
+	g_assert_cmpint(jcat_bits_length64(7), ==, 3);
+	g_assert_cmpint(jcat_bits_length64(16), ==, 5);
+	g_assert_cmpint(jcat_bits_length64(64), ==, 7);
+	g_assert_cmpint(jcat_bits_length64(0x8000000000000000), ==, 64);
+	g_assert_cmpint(jcat_bits_length64(0xFFFFFFFFFFFFFFFF), ==, 64);
+}
+
+static void
+jcat_bt_verifier_func(void)
+{
+	GBytes *buf;
+	g_autofree gchar *fn = NULL;
+	g_autofree gchar *str = NULL;
+	g_autoptr(GBytes) blob = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(JcatBtVerifier) btverifier = NULL;
+
+	fn = g_test_build_filename(G_TEST_DIST, "test.btverifier", NULL);
+	blob = jcat_get_contents_bytes(fn, &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(blob);
+
+	btverifier = jcat_bt_verifier_new(blob, &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(btverifier);
+
+	str = jcat_bt_verifier_to_string(btverifier);
+	g_print("%s\n", str);
+
+	g_assert_cmpstr(jcat_bt_verifier_get_name(btverifier), ==, "lvfsqa");
+	g_assert_cmpstr(jcat_bt_verifier_get_hash(btverifier), ==, "c463f084");
+	g_assert_cmpint(jcat_bt_verifier_get_alg(btverifier), ==, 1);
+	buf = jcat_bt_verifier_get_key(btverifier);
+	g_assert_cmpint(g_bytes_get_size(buf), ==, 32);
+}
+
+static void
+jcat_bt_checkpoint_func(void)
+{
+	GBytes *buf;
+	g_autofree gchar *fn = NULL;
+	g_autofree gchar *str = NULL;
+	g_autoptr(GBytes) blob = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(JcatBtCheckpoint) btcheckpoint = NULL;
+
+	fn = g_test_build_filename(G_TEST_DIST, "test.btcheckpoint", NULL);
+	blob = jcat_get_contents_bytes(fn, &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(blob);
+
+	btcheckpoint = jcat_bt_checkpoint_new(blob, &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(btcheckpoint);
+
+	str = jcat_bt_checkpoint_to_string(btcheckpoint);
+	g_print("%s\n", str);
+
+	g_assert_cmpstr(jcat_bt_checkpoint_get_origin(btcheckpoint), ==, "lvfsqa");
+	g_assert_cmpstr(jcat_bt_checkpoint_get_identity(btcheckpoint), ==, "lvfsqa");
+	g_assert_cmpstr(jcat_bt_checkpoint_get_hash(btcheckpoint), ==, "c463f084");
+	g_assert_cmpint(jcat_bt_checkpoint_get_log_size(btcheckpoint), ==, 4);
+	buf = jcat_bt_checkpoint_get_pubkey(btcheckpoint);
+	g_assert_cmpint(g_bytes_get_size(buf), ==, 32);
+	buf = jcat_bt_checkpoint_get_signature(btcheckpoint);
+	g_assert_cmpint(g_bytes_get_size(buf), ==, 64);
+	buf = jcat_bt_checkpoint_get_payload(btcheckpoint);
+	g_assert_cmpint(g_bytes_get_size(buf), ==, 54);
+}
+
+static void
+jcat_bt_common_func(void)
+{
+	gboolean ret;
+	g_autofree gchar *fn_btcheckpoint = NULL;
+	g_autofree gchar *fn_btverifier = NULL;
+	g_autoptr(GBytes) blob_btcheckpoint = NULL;
+	g_autoptr(GBytes) blob_btverifier = NULL;
+	g_autoptr(JcatBtCheckpoint) btcheckpoint = NULL;
+	g_autoptr(JcatBtVerifier) btverifier = NULL;
+	g_autoptr(JcatContext) context = jcat_context_new();
+	g_autoptr(JcatEngine) engine = NULL;
+	g_autoptr(JcatResult) result = NULL;
+	g_autoptr(GError) error = NULL;
+
+	fn_btverifier = g_test_build_filename(G_TEST_DIST, "test.btverifier", NULL);
+	blob_btverifier = jcat_get_contents_bytes(fn_btverifier, &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(blob_btverifier);
+
+	btverifier = jcat_bt_verifier_new(blob_btverifier, &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(btverifier);
+
+	fn_btcheckpoint = g_test_build_filename(G_TEST_DIST, "test.btcheckpoint", NULL);
+	blob_btcheckpoint = jcat_get_contents_bytes(fn_btcheckpoint, &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(blob_btcheckpoint);
+
+	btcheckpoint = jcat_bt_checkpoint_new(blob_btcheckpoint, &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(btcheckpoint);
+
+	/* get engine */
+	engine = jcat_context_get_engine(context, JCAT_BLOB_KIND_ED25519, &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(engine);
+	g_assert_cmpint(jcat_engine_get_kind(engine), ==, JCAT_BLOB_KIND_ED25519);
+	g_assert_cmpint(jcat_engine_get_method(engine), ==, JCAT_BLOB_METHOD_SIGNATURE);
+
+	ret = jcat_engine_add_public_key_raw(engine, jcat_bt_verifier_get_key(btverifier), &error);
+	g_assert_no_error(error);
+	g_assert_true(ret);
+
+	// TODO: check jcat_bt_checkpoint_get_origin == jcat_bt_verifier_get_name
+	// TODO: check jcat_bt_checkpoint_get_identity == jcat_bt_verifier_get_name
+	result = jcat_engine_pubkey_verify(engine,
+					   jcat_bt_checkpoint_get_payload(btcheckpoint),
+					   jcat_bt_checkpoint_get_signature(btcheckpoint),
+					   JCAT_VERIFY_FLAG_NONE,
+					   &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(result);
+}
+
 int
 main(int argc, char **argv)
 {
+	g_setenv("G_TEST_SRCDIR", SRCDIR, FALSE);
+	g_setenv("G_TEST_BUILDDIR", BUILDDIR, FALSE);
 	g_test_init(&argc, &argv, NULL);
 
 	/* only critical and error are fatal */
 	g_log_set_fatal_mask(NULL, G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL);
 	g_setenv("G_MESSAGES_DEBUG", "all", TRUE);
 
+	g_test_add_func("/jcat/bits", jcat_bits_func);
 	g_test_add_func("/jcat/blob", jcat_blob_func);
 	g_test_add_func("/jcat/item", jcat_item_func);
 	g_test_add_func("/jcat/file", jcat_file_func);
+	g_test_add_func("/jcat/bt-verifier", jcat_bt_verifier_func);
+	g_test_add_func("/jcat/bt-checkpoint", jcat_bt_checkpoint_func);
+	g_test_add_func("/jcat/bt-common", jcat_bt_common_func);
 	g_test_add_func("/jcat/engine{sha1}", jcat_sha1_engine_func);
 	g_test_add_func("/jcat/engine{sha256}", jcat_sha256_engine_func);
 	g_test_add_func("/jcat/engine{gpg}", jcat_gpg_engine_func);
@@ -957,5 +1181,6 @@ main(int argc, char **argv)
 	g_test_add_func("/jcat/context{verify-blob}", jcat_context_verify_blob_func);
 	g_test_add_func("/jcat/context{verify-item-sign}", jcat_context_verify_item_sign_func);
 	g_test_add_func("/jcat/context{verify-item-csum}", jcat_context_verify_item_csum_func);
+	g_test_add_func("/jcat/context{verify-item-target}", jcat_context_verify_item_target_func);
 	return g_test_run();
 }
